import { Listener, EventName } from './event-system';
export declare type Step<Info> = Info & {
    duration: number;
};
export declare type TimerState = 'playing' | 'stopped' | 'paused';
export declare type TimerListener<Event extends EventName<TimerEventsMap>> = Listener<TimerEventsMap[Event]>;
export interface TimerEvent<Info extends object = {}> {
    /** Referece to the timer */
    timer: Timer<Info>;
    /** Elapsed time for current step */
    elapsed: number;
    /** Elapes time from the first step */
    totalElapsed: number;
    /** Current step */
    step: Step<Info>;
    /** Current state of the timer */
    state: TimerState;
    /** Index of the current step */
    stepIndex: number;
}
export interface TimerEventsMap<Info extends object = {}> {
    start: TimerEvent<Info>;
    pause: TimerEvent<Info>;
    end: TimerEvent<Info>;
    tick: TimerEvent<Info>;
    stepchange: TimerEvent<Info>;
}
export interface Timer<Info extends object = {}> {
    /** Return all steps */
    steps(): Array<Step<Info>>;
    /** Set new steps */
    steps(steps: Array<Step<Info>>): Timer<Info>;
    /** Add a step */
    pushStep(step: Step<Info>): Timer<Info>;
    /** Return tick rate */
    tickRate(): number;
    /** Set tick rate */
    tickRate(rate: number): Timer<Info>;
    /** Return tick duration */
    tickDuration(): number;
    /** Set tick duration */
    tickDuration(rate: number): Timer<Info>;
    /** Return the current running step */
    activeStep(): Step<Info>;
    /** Return the state of the timer */
    state(): TimerState;
    /** Start the timer */
    start(): void;
    /** Pause the timer */
    pause(): void;
    /** Add event listener */
    on<E extends EventName<TimerEventsMap>>(event: E, listener: TimerListener<E>): void;
    /** Remove event listener */
    off<E extends EventName<TimerEventsMap>>(event: E, listener: TimerListener<E>): void;
}
export declare function multiTimer<Info extends object = {}>(): Timer<Info>;
